#pragma rtGlobals=1		// Use modern global access method.



//										Last Changes     April/04/02
//This Macro-file includes procedures for Deconvolution, Noise Analysis, and Simulations, as used by 
//Sakaba and Neher in the following papers:  
//Neher & Sakaba, 2001 Combinig Deconvolution....  J.Neurosci. 21:444-461 (referred to as N&S 2001a)
//Sakaba & Neher, 2001 Quantitat. Relationship......  J.Neurosci. 21:462-476 (referred to as S&N 2001)
//Neher & Sakaba, 2001 Estimating Transm. release..  J. Neurosc. 21:9638-9654 (referred to as N&S 2001b)

//The Macros for noise analysis and for the filtering of traces are those used  in  N&S 2001b;  note that 
//N&S 2001a and  S&N 2001 use a slightly different filtering procedure based on differentiation of the 
//experimental traces.  The two procedures give very similar results.
//N&S 2001a and  S&N 2001 also give mEPSC  amplitude estimates  in relative units, whereas  the current
//Macros, given here,  return calibrated values, as described in  N&S 2001b

//The Macros were usually run under IGOR Version 4.01 on a  Power MacIntosh G3;  some of the Macros use 
//Global Variables , as explained in the Macro MakeGlobals; some other global variables and and 
//globally defined waves are explained in the  headers of the Macros

//Some general conventions:
//mEPSCs are specified by a rise time (the global 'risetime', a fast decay timeconstant Tau1(in seconds),
//the variable 'fraction ' (relative contribution ofa slow decay component) and a slow timeconstant Tau2(sec)
//according to  Mini = (1-alpha)*exp(-x/Tau1) + alpha*exp(-x/Tau2) -exp(-x/risetime)
	// 
//Many of the  Macros operate on several traces, as specified by a list of numbers.
// See explanations in the file 'Readme' on how these lists and the arguments of these Macros
// such as ....prefix, postfix1,newpostfix,ListName....  are used to define these traces
	//
//Some Macros use waves which are the results of calculations by other Macros;  therefore they have to be called 
//in  a certain order and the name conventions (names are  defined  automatically within the macros) have to be
//be obeyed  



Macro MakeGlobals ()
		//This creates all the global Variables, which are used in various places
		//Note, that the mEPSC-parameters Tau1, alpha, and Tau2 are arguments of all the Macro-Calls, which
		//use them, although they are used like global parameters;  the mini-risetime, however is a global 

	//strings used for forming wave-names
string/G prefix = "wave_1_"
string/G postfix1 =  "_1"
string/G postfix2 =  "_1_2nd"
	// Variables for noise analysis
Variable/G  ChanAmp = -3e-14		//the effective single channel ampl. i' (after filtering!) in Amps
Variable/G  MiniVar = 5.08e-23	//Variance at a rate of  1event /msec in Amps^2This value and the 
Variable/G Hs = 2.766				//following ones are being reset in AbsCalibration, 
Variable/G Zs = 2490				//see 'AbsCalibration' for their meaning
Variable/G H4 = 1.941
Variable/G Z4 =  7206
	// Variables used for filtering of original traces and describing  experimental conditions
Variable/G T_h = 0.0003			//The filtering window for the highpass in seconds
Variable/G T_1 = 0.0003			////The filtering window for the lowpass in seconds
Variable/G Window = 0.005		//length of the 'gliding window' used for smoothing in the calculation of
								//average Variance, skew etc. ,  in seconds
Variable/G V_hold = -0.08, V_reversal = 0.01//  Holding potential and reversal pot. of AMPA-currents

	// Variables for the diffusion model and deconvolution
Variable/G D= 30				//diffusion coefficient of glutamate in um^2/sec
Variable/G dist = 0.8			//diffusional distance in um;  denoted by r_D in  N&S (2001)
Variable/G	n = 1.2				//The exponent of the postsynaptic power function
Variable/G	nd = 0.9			//The exponent of the diffusion law
Variable/G SmoothDecon = 0.0002	//Smoothing of the deconvol. in sec.
Variable/G	m = 24				//The compression factor  for faster calculatiuon of the residual glu 
Variable/G riseTime = 0.0002	//...of mEPSC in sec

End


Macro PrintGlobals ()
		//This prints all the global Variables
Print "Namestrings: "
Print  "prefix = ", prefix, "   postfix1 = ", postfix1,"   postfix2 = ", postfix2

Print   " "
Print "Clibration parameters: "
Print  "ChanAmp = ", ChanAmp,"   MiniVar = ", MiniVar,"    Hs =", Hs ,"    Zs =", Zs 
Print "H4 =", H4 ,"   Z4 =", Z4
Print " "
Print "Filtering and  potentials: "
Print  "T_h =  ",T_h ,"   T_1 =  ",T_1
Print "Window  =  ",Window
Print  "V_hold = ", V_hold, "  V_reversal = ",V_reversal, " Volt"
Print  " "
Print  "Parameters of the diffusion model and Deconvolution:"
Print "D=  ",D
Print "Diffusion dist =  ",dist
Print	"n =  ",n//The exponent of the postsynaptic power function
Print	"nd =  ",nd//The n of the diffusion law
Print "SmoothDecon =  ",SmoothDecon,"  m =  ",m	//The compression factor  for calculating Int3
Print "Mini riseTime =  ",riseTime
End


// Many of the following Macros operate on several traces, as specified by a list of numbers.
// See explanations in the file 'Readme' on how these lists and the arguments of these Macros
// such as ....prefix, postfix1,newpostfix,ListName....  are used to define these traces

Macro RunSerCorrList (prefix, postfix1,newpostfix,ListName,ResList,CapList,fraction)
	string prefix, postfix1, newpostfix,ListName,ResList,CapList
	Variable fraction
		// see  Function  'SeriesResComp' for an explanation of R_s compensation;
		//  this Macro does a series resistance correction to all the waves in the list;
		//  it saves corrected waves with newpostfix 'Corr' and takes values for Serresistance and  
		// 	capacitance from lists into which values for each trace must  have been entered
		//  in Ohms and Farads
	string RawName, CorrName, N1
	Variable is = 0
	Do
		N1 =  num2iStr($ListName[iS])  
		RawName = PreFix + N1 + PostFix1
		CorrName = PreFix + N1 + NewPostFix
		Duplicate/O $RawName, $CorrName
		RunSeriesResComp ($RawName, $CorrName,$ResList[is],$CapList[is],fraction)		
		iS += 1
	While (iS < numpnts($ListName))

End


Macro RunPreviewList (prefix, postfix1, postfix2,ListName)
	string prefix, postfix1, postfix2,ListName
			// Makes an overlay of all  the waves on the list; also forms averages of both pre- and post-
			//synaptic current traces and returns them with names ListName +"Av" and ListName +"Av2"
			
	string Y1Name, Y2Name, N1,CoName= "Colour"
	string MeanName = ListName +"Av", Mean2Name = ListName +"Av2"
	Variable is = 0,  i=0
	Make/O/N=3 $CoName		//Color name strings!
	$CoName[0] = 60001;$CoName[1] = 1;$CoName[2] = 1
		N1 =  num2iStr($ListName[iS])  
		Y1Name = PreFix + N1 + PostFix1
		Y2Name = PreFix + N1 + PostFix2
		Duplicate/O $Y1Name $MeanName,  $Mean2Name
		Display/R $Y1Name
		ModifyGraph rgb($Y1Name)=($CoName[0],$CoName[1],$CoName[2])  //red, green, blue
		Label bottom "Time  msec"
		If (cmpstr(postfix2 , ""))
			AppendToGraph/R $Y2Name
			ModifyGraph rgb($Y2Name)=($CoName[0],$CoName[1],$CoName[2])
			$Mean2Name  = $Y2Name
		endif
		ModifyGraph zero(right)=8
		SetAxis right -1.5e-08,2e-9; DelayUpdate
		Label right "Current A"
 		//SetAxis bottom 0,0.33; DelayUpdate
		Legend/N=text0/F=0/A=MC
		ShowInfo
		$MeanName = 	$Y1Name
		is +=1	
		i +=1

	
	Do
		N1 =  num2iStr($ListName[iS])  
		Y1Name = PreFix + N1 + PostFix1
		Y2Name = PreFix + N1 + PostFix2
		//print i,Y1Name
		$MeanName += 	$Y1Name
		i +=1	
		AppendToGraph/R $Y1Name
		If (cmpstr(Postfix2 , ""))
			AppendToGraph/R $Y2Name
			ModifyGraph rgb($Y2Name)=($CoName[0],$CoName[1],$CoName[2])
			$Mean2Name  += $Y2Name
		endif
		ChangeColour ($CoName)
		ModifyGraph rgb($Y1Name)=($CoName[0],$CoName[1],$CoName[2])
	iS += 1
	While (iS < numpnts($ListName))
	$MeanName /= i
	$Mean2Name /= i
		AppendToGraph/R $MeanName
		ModifyGraph rgb($MeanName)=(65535,16385,55749)
End


//	Here starts the swection on Deconvolution


Macro DeConvol___________()
		//just a trick to create a mark in the  Macro Menu
	Print "This is not a Macro, but only a spacer "
End



Macro StartGraph (IPName,ICaName, Flag)
	string IPName,ICaName
	Variable flag
			//The Macro creates a Graph with the EPSC, the Ca-current, the Variance of the EPSC. 
			//For Variance a new wave $VName is made  
			//The display of the Variances is such that their scaling agrees with that of release rate
			//For this purpose 2 scaled waves, $VDName , $VCDName are created
			//The input parameters are the names of the current traces (which must exist)
			//Flag:  If 0, a VarTrace is assumed to exist with name "V"+IPName; If flag= 1  Variance 
			//is calculated by NVar ($VName), and corrected  for channel noise ($VCName)
	
	Variable endval= rightx($IPName)

	string VName,VCName, VDName,VCDName
	VName = "V" + IPName		// Variance trace
	VCName = "VC" + IPName	// ChannelVariance trace; same name made within RunVarN
	VDName = "VD" + IPName	// Variance trace; for Display (scaled tp match release)
	VCDName = "VCD" + IPName	// Channel Variance trace; for Display (scaled tp match release)

	If (Flag ==1)
	RunVarN (IPName,"add",T_h, T_1,0,endval) //This creates 2 waves: Fitered trace and Variance
	ENDIF

	Duplicate/O $VName $VCName, $VDName , $VCDName
	$VCName = ChanAmp*$IPName	//the trace for Channelnoise V_c
			//Make the Graphs
	$VDName = ($VName - $VCName)/MiniVar	//release rate, as estimated from Variance
	$VCDName =$VCName/MiniVar	//Channel variance at same scaling as  other Variance trace
	Display/R $IPName
	AppendToGraph/L $VDName,$VCDName
	ModifyGraph lstyle($VDName)=1,rgb($VDName)=(13102,26214,0)
	ModifyGraph lstyle($VCDName)=2,rgb($VCDName)=(13102,26214,0)
	Label bottom "Time  sec"
	AppendToGraph/R $ICaName
	ModifyGraph rgb($ICaName)=(0,2,26214)
	ModifyGraph zero(right)=8
	SetAxis right -1.5e-08,2e-9; DelayUpdate
	SetAxis left -10,100; DelayUpdate
 	SetAxis bottom 0,0.33; DelayUpdate
	Legend/N=text0/F=0/A=MC
	ShowInfo
end



Macro RunStartList (prefix, postfix1, postfix2,ListName)
	string prefix, postfix1, postfix2,ListName
			
			//  Makes all the startgraphs of the waves on the list  including the variance traces
			// see ReadMe text for the  meaning of the strings 
	string Y1Name, Y2Name, N1,
	Variable is = 0
	Do
		N1 =  num2iStr($ListName[iS])  
		Y1Name = PreFix + N1 + PostFix1
		Y2Name = PreFix + N1 + PostFix2
		StartGraph (Y1Name,Y2Name,1)
		Print  "StartGraph of trace:  ", Y1Name
		iS += 1
	While (iS < numpnts($ListName))

End



Macro RunTemplateList (prefix, postfix1, postfix2,ListName,EarlySlope,alpha,tau2,q,beta,Flag)
	string prefix, postfix1, postfix2,ListName
	Variable EarlySlope,alpha,tau2,q,beta,Flag
			//Special routine for analyzing mean EPSC (deconvolution) and mean-Variance of a list of 
			//'fitting protocols',  Variance is calculated from difference records, with the advantage that 
			//transients are less harmful.	
			//prameters are those of RunStartList and CalcRelease combined; works on post- ane presynaptic 
			// current records;  makes a overview graph of the waves on the list and calculates Average_2nd-trace
			//Then,  the Average-Variance of EPSCs is calculated  and displayed together with  meanEPSC etc 
			//and deconvolution performed;  
			//If only very few traces are available, the first trace of the list should be repeated at the end 
			//of the list. This will include a difference (last_trace - first_trace ) in the A-Var calculation
			//Flag:  same meaning as that of  CalcRelease (3:  Display;  1  not)
	RunPreviewList (prefix, postfix1, postfix2,ListName)
	string Mean2Name = ListName +"Av2"	//These are the names of the 
			//average waves generated in RunPreviewList
	string VCName
	AvVar (PreFix,PostFix1, listName,ListName,0)	//Flag 0 avoids calculating mean rates etc
			//Here the average Y-value and the Average  Variance is calculated and dislayed
	string MeanName ="Av"+ ListName		//This is the name of the 
			//average wave generated in AvVar
	Variable endval = rightx($MeanName)
	VCName = "VC" + MeanName
	Duplicate/O $MeanName $VCName
	$VCName = ChanAmp*$MeanName	//This is channel variance V_c
	
	StartGraph (MeanName,Mean2Name, 0)

	CalcRelease (MeanName,Mean2Name,EarlySlope,alpha,tau2,q,beta,0.004,endval,Flag)

End




Macro AddReleaseTraces (preFix, postfix1, number)
	string preFix, postfix1	//these strings and 'number' define the trace-name
	Variable number
		//adds the traces calculated in CalcRelease to  the  top graph; it is assumed that  a decon-
		//volution has already been performed,either by running 'CalcRelease' or 'RunCalcList'
		//this is particularly useful after 'RunCalcList', because the latter creates startgraphs 
		//and performs deconvolutions, but does not display the results
	string MName,NName, IPname,RName, IName
	IPName = prefix+ num2iStr(number)  + postfix1
	MName =  "M" + IPName
	NName =  "N" + IPName
	IName =  "I" + IPName
	RName =  "R" + IPName
		AppendToGraph $MName
		AppendToGraph/L=NAxis $NName	//the N-trace (remaining Pool) needs its own axis
		SetAxis NAxis -200,2000			//about  10 times the range of left axis
		ModifyGraph lstyle($NName)=2,rgb($NName)=(39321,26208,1);DelayUpdate
		Label NAXIS " Pool Size  #";
		Label left " Release RATE  #per msec";
		ModifyGraph zero(left)=8
		AppendToGraph/R $IName
		AppendToGraph/R $RName
		ModifyGraph rgb($RName)=(0,43690,65535)
		SetAxis left -10,100; DelayUpdate
		RemoveFromGraph $IPName
End



Macro CalcRelease (IPName,ICaName,EarlySlope,alpha,tau2,q,beta,t_start,t_test,Flag)
	string IPName,ICaName
	Variable EarlySlope,alpha,tau2,q,beta,t_start,t_test,Flag
	//This is the Macro performing the Deconvolution, as described in N&S 2001a
	// IPName,ICaName  are names of postsynaptic current and presynaptic Ca-current wave
	//EarlySlope is the initial descending slope of the mEPSCin sec^-1. This, together with alpha and 
	//Tau2   is used to calculate  Tau1 (see general conventions about the mEPSC above) 
	// q: miniamplit. relative to the mean of the mEPSC amplitude distribution (which must be 
	// provided as the wave 'StandardMini' (q usually has negative sign!); beta is a scaling factor
	//for the residual current (all other parameters of the diffusion model are global parameters).
	// t_start:  starttime of analysis; before that time the average of postsyn current is measured
	//  and subtracted from the vuve (baseline subtraction); t_test  time at which vesicles are 
	// for a first order calculation of the poolsize (= cumulative release)
	//  Use Flag =3  for the first run of the Macro on a given trace and for displaying the results
	//  Use Flag = 2 for new traces without display (such as in RunCalcList)
	//  Use Flag =1  for second and third runs on a given trace; no new display
	// Flag = 5 calculates reconstruction from  'TestRate';  use only for simulation prposes
	//Waves $IPName,$ICaName,$VCName  have to exist
	//The Macro creates and displays the relesae rate ($MName), remaining vesicles ($NName)
	//and residual current ($RName)

	Variable Tau1, N_t, MiniSize
	Tau1 = (1- alpha)/(EarlySlope - alpha/tau2)
	Print  "Tau1  ", Tau1*1000,"msec	"
 
	string MName,NName, Iname,RName,VCName

	MName = "M" + IPName
	NName =  "N"  + IPName
	IName =   "I"   + IPName
	RName =  "R"  + IPName
	VCName = "VC" + IPName

	If (Flag >1)		//Flag < 2 for recalling the macro repetitively 
		Duplicate/O $IPName $Iname, $MName, $NName ,$RName
		//waves for leak subtracted current; release rate(M); remaining vesicles(N)
		// and reconstruction of residual current(R)
	Endif
	
	MiniSize =q*MeanFromHist (StandardMini) 	//  The histogram 'Standardmini' has to exist 
				//with proper x-scaling Bin width in Amp; see function MeanFromHist;  
	Print  "MiniAmplitude h: = ",  MiniSize,"     RelMiniSize = ", q

		//  Do the deconvolution here
	SubOffset ($Iname, t_start)	// correct baseline, such that the mean between 0 and t_start is zero
	Decon ($Iname,$MName,$RName,Tau1,alpha,tau2,beta,t_test,Flag)//returns result in $MName

		// work-up the deconvolution result
	$NName = $MName
	Duplicate/O $MName TestRate//  a trick such that the Macro can also beused for simulations

		//work on the release rate
	$MName = $MName/(MiniSize*1000)	//rate inunits of release eventsper msec, assuming 
											//uniform mini amplitudes
	DoubleSmooth ($MName, SmoothDecon)	//smooth the result with global variable (in sec)

		//now get rough estimate of the size of the pool
	Integrate $NName	//The integral of the rate
	$NName = $NName/MiniSize	//this is now the integral of M (the total No of  vesicles released )
	N_t = Calcmean ($NName, t_test-0.005,t_test,0)	// mean over 5msec window starting at t_test
	$NName = N_t  - $NName
	Print "Pool size =  ", N_t,"   as measured at  ", T_test,"sec +2.5msec"
	
	//$VCName = 0
	//$VCName = ChanAmp*$IPName   // this is to recalculate the channel noise on the basis of the
	// whole postsynaptic current;  needed for 'ReleaseFromWindow', but already calculated in Startgraph
	
	If (Flag ==3)		//add the new traces to Graph
		//$VCName = ChanAmp*$RName   // this is to update the channel noise on the basis of the reconstruction
		//Print "Channel noise updated based on Reconstruction and  ChanAmp0 ", ChanAmp
		AppendToGraph $MName
		AppendToGraph/L=NAxis $NName	//the N-trace (remaining Pool) needs its own axis
		SetAxis NAxis -200,2000			//about  10 times the range of left axis
		Label NAXIS " Pool Size  #";
		ModifyGraph lstyle($NName)=2,rgb($NName)=(39321,26208,1);DelayUpdate
		Label left " Release RATE  #per msec";
		ModifyGraph zero(left)=8
		AppendToGraph/R $IName
		RemoveFromGraph $IPName
		AppendToGraph/R $RName
		ModifyGraph rgb($RName)=(0,43690,65535)
		//SetAxis left -5,200
	EndIf
end



Macro CalcPool(IPName, t_empty, TauRecover,N_tFrac)
	string IPName
	Variable t_empty, TauRecover,N_tFrac
			//This Macro calculates  a selfconsistent NWave (for Pool size as a function of time)
			// and N_t (Total Pool Size) under the assumtion that the pool is empty at 
			//t_empty but also recovers while releasing with  recovery time 'TauRecover' according
			//to  S&N2001, p. 463.  Pool sizes are evaluated in windows [2msec,4msec] and
			//[t_empty-0.005,t_empty]
			//Also, it calculates the release rate per remaining vesicle (wave
			//  $PName) for all data points, for which the remaining pool is  larger than 'N_tFrac' 
			//IPName is the name of the EPSC; a deconvolution must have been performed  before
			//...converges (and is valid) only for short stimulation protocols (relative to TauRecover)
			//Note, that this procedure approximates the pool by a homogeneous one

	string MName,NName,PName

	MName = "M" + IPName
	NName = "N" + IPName
	PName = "P" + IPName
	Variable i = 0, upto = 10, N_t, N_tnew, N_told

	Duplicate/O $MName $PName
	$PName =1000*$MName	// use PName temporarily as M-integral
	Integrate $PName
	N_t =-Calcmean ($PName, 0.002,0.004,0)+Calcmean ($PName, t_empty-0.005,t_empty,0)	
	$NName =  N_t - $PName
	//$NName - Calcmean ($NName, t_empty-0.005,t_empty,0)
	Duplicate/O $NName TempInt

	Do
		N_told = N_t
		TempInt = $NName
		Integrate TempInt
		$NName = N_t*(1+x/TauRecover)- $PName - TempInt/Taurecover
		N_tnew = Calcmean ($NName, 0.002,0.004,0)-Calcmean ($NName, t_empty-0.005,t_empty,0)
			//difference of means :  3msec to 5msec before t_empty
		N_t = 0.7*N_tnew + 0.3*N_t
		PRINT  N_told, N_tnew
		i+=1
	while (i<upto)
	
	$PName =  $MName/$NName	//the rate of release per remaining vesicle
	Blank ($PName, $NName, N_tFrac)	// set values of  $PName to NAN if $NName(x) < N_tFrac
end




Macro RunDuplicationList (Oldprefix, postfix1, postfix2,ListName,NewPreFix)
	string Oldprefix, postfix1, postfix2,ListName,NewPreFix
			
			// Duplicates  all the Post and presynaptic current waves  on the list
			// This is useful, if a second analysis is supposed to be performed  within the same IGOR
			// experiment to avoid overwriting of old resultwaves
	string Y1Name, Y2Name, N1
	string Y3Name, Y4Name
	Variable is = 0
	Do
		N1 =  num2iStr($NumWName[iS])  
		Y1Name = OldPreFix + N1 + PostFix1
		Y2Name = OldPreFix + N1 + PostFix2
		Y3Name = NewPreFix + N1 + PostFix1
		Y4Name = NewPreFix + N1 + PostFix2
		Duplicate/O $Y1Name $Y3Name
		Duplicate/O $Y2Name $Y4Name
		Print  "Waves created by duplication:  ", Y3Name, "    ",Y4Name
		iS += 1
	While (iS < numpnts($NumWName))
		prefix = NewPreFix
		Print "Global Prefix changed to:  ", Newprefix 
End


Macro RunCalcList (ListName,Earlyslope,alpha,tau2,q,beta,t_start,t_test)
	string ListName
	Variable Earlyslope,alpha,tau2,q,beta,t_start,t_test
			//Performs the Deconvolution on a whole list of traces 'ListName' . Define pre- and
			// Postfixes, such that together with the list, they give proper names ;the other 
			//  parameters are so those of 'Calcrelease'
			//The Macro does no displays; For Displays run 'RunStartList' and use 'AddReleaseTraces' 
	string Y1Name, Y2Name, N1,
	Variable is = 0
	Do
		N1 =  num2iStr($NumWName[iS])  
		Y1Name = PreFix + N1 + PostFix1
		Y2Name = PreFix + N1 + PostFix2
		Print  "			CalcRelease of trace:  ", Y1Name
		CalcRelease (Y1Name,Y2Name,Earlyslope,alpha,tau2,q,beta,t_start,t_test,2)//do the Deconvol.
		iS += 1
	While (iS < numpnts($NumWName))

End

Macro RunAllDup ( list,newprefix,ResFix,Earlyslope,alpha,tau2,q,beta,t_start,t_end)
	string  list, newprefix,ResFix
	Variable Earlyslope,alpha,tau2,q,beta,t_start,t_end
		//Duplicat a complete analysis
	RunDuplicationList (prefix, postfix1, postfix2,list,newprefix)	// Duplicate current records
	RunStartList (prefix, postfix1,postfix2,list) 					//Create the graphs
	RunCalcList (list,Earlyslope,alpha,tau2,q,beta,t_start,t_end,1)	//Do the deconvolution
	AvVar (prefix, postfix1, list,resFix,1)							//Calculate Average variance
End


// Here starts the section on noise analysis



Macro Noise_____________()
	Print "This is not a Macro, but only a spacer in the Macro List"
End



Macro RunVarN (TraceName,NameMode,DelTa, Fil1,t_start,t_end)
	string TraceName, NameMode
			// Runs  the  Variance calculation by preparing all the necessary waves and calling  CalcVarN
			//TraceName: Name of record to beanalyzed
			//NameMode:  if "replace"  the first letter of tracename will be replaced by  'V' and 'F' to
			//make names for the result waves Variance and filtered traces; otherwise letters are added;
			//data are bandpassfiltered with lowpassfilterwindow 'Fil1' and  highpass filterwindow 'Delta'
			//the result is smoothed with global variable 'window'
	Variable DelTa, Fil1,t_start,t_end, WindowP,WindowP2
	
	string  VName, FName
	IF (CmpStr ( NameMode,"replace"))
		VName = "V" + TraceName
		FName = "F" + TraceName
	ELSE
		 VName = TraceName; FName =  TraceName
		VName[0,0] = "V" 	//Replace 1st letter by "V"  
		FName[0,0] = "F" 	//Replace 1st letter by "F"  
	EndIf

	
	Duplicate/O $TraceName $VName, $FName
	
		CalcVarN ($TraceName,$VName,$FName,DelTa,Fil1,t_start,t_end)	//do the variance calculation
		DoubleSmooth ($VName, window)			//smooth the result
end



Macro AvVar (PreFix,PostFix, ListName,ResultPostFix,Flag)
	string PreFix,PostFix, ListName,ResultPostFix
	Variable Flag
			//Calculates average variance and current from difference traces, as listed in the 
			//Wave "ListName", not corrected for  Channel noise
			// The average M- and N-Wave are also calculated  if Flag = 1(estimates for release rates
			// and  poolsizes, as determined in a previuosly performed Deconvolution)
			//Contrary to AvCumulants, this Macro bases its estimates on pairwise-difference-traces
			//Traces are bandpassfiltered (in CalcVarN) and the result is smoothed with 'window'
	string AvVName, AvYName, AvMName,AvNName,YName,N1,N2
	Variable  SumN= 0 	//We need these as globals, because we do not have enough
											//Variables to pass to  'AddDiffVariance
	Variable t_end = 0.5, iS=0,No1,No2
	AvVName = "VAv" + ResultPostFix
	AvYName = "Av" + ResultPostFix
	AvMName = "MAv" + ResultPostFix
	AvNName = "NAv" + ResultPostFix
	N1 =  num2iStr($ListName[0])  
	YName = PreFix + N1 + PostFix
	//Print yname
	Duplicate/O $YName $AvVName, $AvYName,$AvMName,$AvNName
	$AvVName= 0
	$AvYName = 0
	IF  (Flag == 1)
		Duplicate/O $YName $AvMName,$AvNName
		$AvMName =0
		$AvNName =0
	EndIf
	
	Do
		No1 = $ListName[is]
		N1 =  num2iStr(No1 )
		No2 = $ListName[is+1]
		N2 =  num2iStr(No2 )
		AddDiffVariance (PreFix,PostFix, N1,N2,AvVName, AvYName, AvMName, AvNName,t_end,Flag)
				//This function adds  Variance-estimates  to AvVName
		SumN += 1
		iS += 1
	While (iS < numpnts($ListName)-1)


	Print "Average of ", sumN,"Difference Traces"
	$AvVName/= SumN
	$AvYName/= SumN
	
				//Smooth with 'window
	DoubleSmooth ($AvVName, window)
	IF  (Flag == 1)
		$AvMName/= SumN
		$AvNName/= SumN
	EndIf

END
	//This was previuosly a display section for AvVar
	Display/R $AvYName
	AppendToGraph/L=VarAxis $AvVName
	ModifyGraph lstyle($AvVName)=1,rgb($AvVName)=(13102,26214,0)
	ModifyGraph lstyle($AvYName)=1,rgb($AvYName)=(39321,1,31457)
	IF  (Flag == 1)
		AppendToGraph $AvMName
		SetAxis left -20,200
	EndIf
	//SetAxis right -1.5e-08,1.5e-09
	//SetAxis bottom 0,0.25
	SetAxis right -1.5e-08,1.5e-09 ;DelayUpdate
	SetAxis bottom 0,0.25 ;DelayUpdate
	SetAxis VarAxis -1e-21,1e-20		//set to  MiniVar (~~5e-23) * LeftAxis
 	Legend/N=text0/F=0/A=MC
	ShowInfo
end




Macro AvCumulants (PreFix,PostFix,ListName,AvPostFix,t_start,t_end,Modeflag,Dispflag,SubFlag,ChanAmp)
	string PreFix,PostFix,ListName,AvPostFix
	Variable t_start,t_end,ModeFlag,DispFlag,SubFlag, ChanAmp
	//Calculates average cumulants using waves from  'ListName' after bandpass filtering
	//according to N&S 2001b;  results are smoothed with Global variable 'window' 
	//Calculates only Variance  if Modeflag =1;  also higher moments  for flag >1
	// In addition:   Rate and Minsize from Skew and Var, if  ModeFlag = 2 
	//			....same , but subtracting channel variance(based on MeanEPSC and 'Chanamp')  if  ModeFlag =3
	// 			 Rate and Minsize from 4th moment  and Skew, if  ModeFlag = 4
	//DispFlag:  One display is created, if 1;  two displays, if 2
	//Creates up to 13 result waves with names, as described below 
	//Subflag, if 1:  meansubtracted traces are used ;  a mean-trace  as calculated in "runPreviewlist" with
	// name list+"Av" must exist or else  AvCumulants must have executed before. The analysis is performed  on
	// the whole trace and the mean is  subtracted with a weight (LSCoefficient)determined such that variations between t_start
	// and t_end are minimized.  LSCoefficient may be set to 1 (below, in the code), if this option is not desired.
	//mEPSC parameters for calibration  according to the general conventions  ( the
	//mEPSC is calculated , based on the amplitude distribution 'Standardmini', the global Variable 'risetime' and 
	// assuming  Tau1, tau2 and alpha to be  0.001,0.012, and 0, respectively)

	
	string TraceName, VName,VCName,SName,FoName,MName,EName,FName,DupName
	string MeanName,MeanVName, MeanSName,MeanFoName,N1,CName
	Variable WindowP,WindowP2,SumN = 0, LSalpha,W_end,Mul1,dum
	N1 = num2iStr($ListName[0])

	TraceName = PreFix + N1 + PostFix	//the name of the first trace to be analyzed; a wave with this name must exist!
	VName = "SIV"  + AvPostFix			//The last Variance trace, overwritten in each loop cycle
	VCName = "SIVC"  + AvPostFix		//...same for Channel variance
	SName = "SIS"  + AvPostFix			//...same for skew
	FoName = "SI4"  + AvPostFix		//...same for 4th moment
	MeanName = "Av" + PreFix + PostFix//Mean of EPSCs
	MeanVName = "Av" + VName			//Mean variance
	MeanSName = "Av" + SName			//Mean Skew
	MeanFoName = "Av" + FoName		//Mean Forth moment;  overwritten by Forth cumulant
	FName = "SIF" + AvPostFix			//The last  high-pass-filtered trace
	MName = "SIM_" +AvPostFix	//The Rate from Cumulants
	EName = "SIE_" + AvPostFix	// The elementary size
	CName = "SIC_" + AvPostFix	// The elementary size
	DupName = "Dup"  + AvPostFix	// The duplicated trace (mean-subtracted)
	string ListMeanName = ListName +"Av"
	
	If  (SubFlag ==1)	//in case of  meansubtraction
	IF (cmpStr(Wavelist(ListMeanName,";",""), ""))	//does  a Meantrace with name ListMeanName already exist
		//here is the case "yes",  a previous $ListmeanName exists (probably from  RunPreviewList)
		Print  "Reuse a previous mean"
	ELSE		//it does not
		IF (cmpStr(Wavelist(MeanName,";",""), ""))	//has  AvCumulants run before and formed a mean
			//yes!
			Duplicate $MeanName $ListMeanName
			Print "Adopt last mean"
		ELSE
				Abort ("No mean trace available")
		ENDIF
	endif
	endif
		
	Duplicate/O $TraceName $DupName,  $VName,$MeanName,$MeanVName, $FName, $VCname
		$MeanName  = 0
		$MeanVName  = 0
		$VCname =0
	IF (Modeflag >1)
		Duplicate/O $TraceName $Sname, $Foname,$MeanSName,$MeanFoName,$MName, $EName, $CName
		Redimension/D $VName
		Redimension/D $MeanVName
		Redimension/D $SName
		Redimension/D $MeanSName
		Redimension/D $FoName
		Redimension/D $MeanFoName
		$MeanSName  = 0
		$MeanFoName  = 0
	Endif

		Print "The first trace to analyze will be:  ",TraceName
		W_end = pnt2x($tracename,numpnts($TraceName)-1)// the x-value of the last point of $TraceName
	IF (DispFlag >0)
		Display $DupName		//will be the  mean subtracted trace later on
		SetAxis left -1e-8,0.5e-8 ;DelayUpdate
	EndIf

	dum = MakeMini ( $TraceName,risetime,0.002,0.01,0)
	AbsCalibration (Mini,standardmini)		//This defines and calculates the global Variables  Hs, Zs,H4, and Z4
											//based on Mini,standardmini, and on the current settings of T_h and T_1


	//  Here starts the averaging loop
	do
		N1 = num2iStr($ListName[SumN])
		TraceName = PreFix + N1 + PostFix	//	a wave with this name must exist!
		If  (SubFlag ==1)
			LSalpha = LSCoefficient ($TraceName, $ListMeanName, t_start,t_end)//minimizing the deviations 
											//in window between  t_start and t_end
			$DupName = $TraceName - LSalpha*$ListMeanName
			//$DupName = $TraceName -$ListMeanName
		ELSE 
			$DupName = $TraceName
			$MeanName  += $DupName		//Calculate Mean-Trace, if meansubtr. is not done
		Endif
		IF (ModeFlag ==1)					//  calculate variance only, using CalcVarN
		CalcVarN ($DupName,$VName,$FName,T_h,T_1,0,W_end)
		$MeanVName  += $VName
		Endif
		If (ModeFlag >1)					//  calculate all cumulants, using CalcCumulants
			CalcCumulants ($DupName,$VName,$SName,$FoName,$FName,T_h,T_1,0,W_end)
			$MeanVName  += $VName
			$MeanSName  += $SName
			$MeanFoName  += $FoName
		EndIf
		SumN +=1
	while (SumN < numpnts($ListName))
	
	// work up the sums:
		Mul1 = SumN/(SumN -1)
		$MeanName  /=SumN
		$MeanVName  /= SumN
		If  (SubFlag ==1)
			$VCName = chanAmp*$ListMeanName
			Print "ChannelVariance based on Mean current from last preview"
		ELSE 
		$VCName = chanAmp*$MeanName
		Endif
		$MeanSName  /= SumN
		$MeanFoName  /= SumN		//We now have mean moments of the difference traces
	Print "Average of ", SumN ,"  Traces"
		DoubleSmooth ($MeanVName, window)
		DoubleSmooth ($MeanSName, window)
		DoubleSmooth ($MeanFoName, window)
		$MeanFoName = ($MeanFoName - 3*($MeanVName)^2)	//replace the mean 4th-moment
					//by the 4th cumulant
		If  (SubFlag ==1)		//Correct for the fact that we have difference traces
			$MeanVName  *= Mul1
			$MeanSName  *=  Mul1^3/(1-(SumN -1)^(-2))
			$MeanFoName  *=  Mul1^4/(1+(SumN -1)^(-3))
		endif

	IF (Modeflag ==3)				//Channel Variance subtraction!
		$MeanVName -= $VCName
		Print "Variance corrected for ChannelVariance, using ChanAmp = ", Chanamp
	ELSE							//otherwise we can calculate i'  (equ. 9 of  N&S, 2001b)
		$CName = $MeanVName/$ListMeanName*(1- $MeanSName^2/$MeanFoName/$MeanVName*Hs/H4)
	ENDIF

	IF (Modeflag ==4)
			//This is calculation using 4th  and 3rd cumulant
			$MName = $MeanSName^4/$MeanFoName^3	*Z4/1000			//  rate in events/msec!	
			$EName = $MeanFoName/$MeanSName*H4				//miniamplitude in Amps
	ELSE
		
		IF (Modeflag >1)
			// This is calculation  with third and second moments
		$MName = $MeanVName^3/$MeanSName^2* Zs/1000			//  rate in events/msec!		
				$EName = $MeanSName/$MeanVName*Hs				// miniamplitude in Amps
		endif
	endif	
		
	IF (DispFlag >0)			//the 1st displays - the mean traces
		IF  (SubFLag ==1)
			AppendToGraph $ListMeanName ;DelayUpdate	// mean EPSC
			ModifyGraph rgb($ListMeanName)=(65535,32768,32768);DelayUpdate
		ELSE 
			AppendToGraph $MeanName ;DelayUpdate
		Endif
		AppendToGraph/R $MeanVName ;DelayUpdate		//mean variance
		AppendToGraph/R $VCName ;DelayUpdate			//channel variance
		AppendToGraph/L=skewaxis $MeanSName ;DelayUpdate	//skew
		Legend/N=text0/F=0/A=MC ;DelayUpdate
		ModifyGraph rgb($MeanVName)=(39321,26208,1),rgb($MeanSName)=(2,39321,1);DelayUpdate
		ModifyGraph lstyle($VCName)=3,rgb($VCName)=(0,26214,13293);DelayUpdate
		ShowInfo;DelayUpdate
		SetAxis right -1e-22,1e-21 ;DelayUpdate
		ModifyGraph zero(right)=3
		SetAxis skewaxis -2e-32,2e-33  ;DelayUpdate
		ModifyGraph zero(skewaxis)=1;DelayUpdate
		//SetAxis bottom 0,W_end ;DelayUpdate
		//SetAxis skewaxis -1e-21,1e-23 ;DelayUpdate
		//SetAxis left -4e-7,2e-7 ;DelayUpdate
		//SetAxis right -1e-15,5e-15 ;DelayUpdate
		Label left "Current  nA";DelayUpdate
		Label bottom "time  s";DelayUpdate
		Label skewaxis "Skew";DelayUpdate
		Label right "Variance"

		//    Add the 4th  cumulant  IF (Modeflag ==4)
			AppendToGraph/R=forthaxis $MeanFoName
			SetAxis forthaxis -2e-44,2e-43
			ModifyGraph rgb($MeanFoName)=(0,6405,26214)
			Label forthaxis "Forth Cum."
		//endif	
	Endif		

	IF (DispFlag ==2)		// a second display 
		Display $EName				//mEPSC amplitude
		AppendToGraph $CName		// i'
		AppendToGraph/R $MName	// release rate
		ModifyGraph rgb($EName)=(0,13107,6646),rgb($MName)=(0,43690,6553)
		ModifyGraph rgb($CName)=(1,20000,19321)
		//SetAxis bottom 0,W_end;DelayUpdate
		SetAxis right -20,20 ;DelayUpdate
		SetAxis left -5e-11,5e-11 ;DelayUpdate
		Label left "mEPSC-Amp.     pA";DelayUpdate
		Label right "Rel. Rate  events/ms"
		ModifyGraph zero(left)=1;DelayUpdate
		ShowInfo;DelayUpdate
		Legend/C/N=text0/F=0/A=MC
	endif

end





// This section contains some general Macros  for measuring and display


Macro Varia_____________()
	Print "This is not a Macro, but only a spacer "
End



Macro MeanFromWindow () 
		//Calculates  the mean in between two cursors, which
		// have to be on same trace in the top Graph
	Variable value
			
	string YName
	
	yName =  CsrWave (A)					//Name of wave that the cursor is on
	If (CmpStr (yName, CsrWave(B)))
		Abort "Both Cursors must be on same wave"
	EndIf

	value = Calcmean ($YName,xcsr(A),xcsr(B),1)
	Print "Analyzing",YName,"     from",xcsr(A)," To ",xcsr(B),"   Mean =  	",value
end
	

Macro MeanFromW_DelT (DelTa)
	Variable DelTa		//length of analysis window in  x-axis units
		//Calculates  the mean of curve with cursor, starting at Crs(A) for the indicated  length 
		//of window

	string YName
		yName =  CsrWave (A)					//Name of wave that the cursor is on

	Variable value,start	
	start = xcsr(A)
	value = Calcmean ($YName,start,start + DelTa,1)
	Print "Analyzing",YName,"     from",start," To ",start + DelTa,"   Mean =  	",value

end




Macro RunCalibration (tracename,Tau1,alpha, tau2)    //
	string tracename
	Variable Tau1,alpha, tau2
			//This Macro calculates calibration constants Hs,Zs,...H4,Z4 for noise analysis as 
			// described in N&S 2001 b  equs 12 - 16
			//To do so it calculates an mEPSC waveform (wave with name 'mini'), as specified  by the
			//arguments Tau1,alpha, tau2, and the global variable risetime (see 'General Conventions
			//above)
			//Tracename:  The name of any wave of the same type as sampled records has to be given; the 
			//Macro will leave this wave untouched ; it uses it just to get its  scaling, length  etc. to create a 
			//mEpsc waveform compatible with it; Calibration constants are calculated
			//from the filtered mini and the amp distribution, which must be supplied as a wave by 
			//name 'StandardMini'. The actual calculation is done  in function 'AbsCalibration'
	Print "Calibration:   calculates a mini, uses the  amp distibution 'standardmini' "
	Print "and the settings of the filter:  T_h =",T_h,"  and lowpass T_1=", T_1
	Variable dum = MakeMini ($tracename,risetime,Tau1,Tau2,alpha)
	AbsCalibration (mini, standardmini)
		//This defines and calculates the global Variables  Hs, Zs,H4, and Z4
End



Macro AddDiffVariance (PreFix,PostFix, Name1,Name2,VarSum, Y2Sum,MSum,NSum,t_end,Flag)
	String PreFix,PostFix, Name1,Name2,VarSum, Y2Sum,MSum,NSum
	Variable t_end, Flag
		//This Macro calculates the Variance of the difference trace (wave2-wave1) and adds:
		// 1: This wave   to  wave 'VarSum'
		// 2: The wave2 to Y2Sum    and
		// 3: (if Flag = 1)   adds up  M-traces and N-traces (which must have been calculated in a
		//previous  deconvolution)
		//Difference traces are bandpassfiltered (in CalcVarN) before calculating Variance
		//The strings PreFix,PostFix, Name1,Name2 define input wave names (see general name conventions)
		//and the strings VarSum, Y2Sum,MSum,NSum are names of waves, which must exist, to which cal-
		//culation results are added.     This Macro is used by the Macro 'AvVar' to  calculate    
		//mean Variance of an ensemble of curves from  pairwise differences

	string  YName1, YName2,MName2,NName2
	//Print name1,name2
		YName1 = PreFix + Name1 + PostFix
		YName2 = PreFix + Name2 + PostFix
		MName2 = "M" +PreFix + Name2 + PostFix
		NName2 = "N" +PreFix + Name2 + PostFix
		Print Name1, Name2,YName2
		Duplicate/O $YName2 AWave,VWave, FWave,Diffwave
		DiffWave = $YName2 - $YName1
		CalcVarN (DiffWave,VWave,FWave,T_h,T_1,0,t_end)
		VWave =VWave/2 	//- ChanAmp*$YName2	   ----   0.5 times the Variance of the Diff.trace
				//previously channel Variance of the second trace was subtracted here already
		$VarSum  += VWave
		$Y2Sum  += $YName2
	IF  (Flag == 1)		//add up for calculating the average M-N-waves
		$MSum  += $MName2
		$NSum  += $NName2
	EndIf
	
end




// Here come some functions, used in the Macros above

	


Function  AbsCalibration (miniwave, ampdist)
	wave miniwave, ampdist

	//absolute calibration parameters for  amplitude and rate estimates
	//needs the normalized(to 1) miniwave , which underlies the simulation or the exper. data
	//and the ampdist (which can be scaled or normalized in both x and y)	
	//defines and calculates the global variables Hs,Zs,...H4,Z4 (equ 11, 12 ..) from  N& S 2001b,
	// using the  global variables T_1 and T_h for filtering.   This Function is usually called
	// via the Macro 'RunCalibration' (see also additional explanations there)


	Variable length1 = (numpnts(MiniWave)-1)
	Variable length = length1*deltax(MiniWave)
	Duplicate/O Mini Minitemplate
	Minitemplate = 0
	Minitemplate [200,length1] = Miniwave[p-200]	

	//Variable length = numpnts(miniwave)-1 //..a previous version 23.1.00
	NVar T_1 = T_1
	NVar T_h = T_h
	IF (T_h > 0.001)
		Print "Abs. Calibration may be wrong due to long Bandpass template"
		//In that case the number 200 (above ) has to be made longer
	EndIf
	Duplicate/O Minitemplate  MIntegral, FMini,F2Mini,F3Mini,F4Mini
	BandFilter (FMini, T_1, T_h)
	F2Mini = FMini^2
	F3Mini = FMini^3
	F4Mini = FMini^4
	Duplicate/O FMini  FMiniInt,F2MiniInt,F3MiniInt,F4MiniInt
	F2MiniInt = f2Mini
	F3MiniInt = f3Mini
	F4MiniInt = f4Mini
	Integrate MIntegral, FMiniInt,F2MiniInt,F3MiniInt,F4MiniInt
	//print FMiniInt(length),F2MiniInt(length),F3MiniInt(length),F4MiniInt(length)
	//Print  Histomoment (ampdist,1), Histomoment (ampdist,2),Histomoment (ampdist,3),Histomoment (ampdist,4)
	//Now work on the  amp-distribution and calculate calibration factors
	Print  "Abs.Calibration based on a standard MiniAmplitude of",Histomoment (ampdist,1)
	Variable/G  Hs,H4,Zs,Z4,MiniVar
	Hs =  F2MiniInt(length)/F3MiniInt(length)	//the integrals include powers of q
	Hs *=  Histomoment (ampdist,1)*Histomoment (ampdist,2)/Histomoment (ampdist,3)
											
	Zs = (F3MiniInt(length))^2/(F2MiniInt(length))^3
	Zs *=  (Histomoment (ampdist,3))^2/(Histomoment (ampdist,2))^3

	H4 =  F3MiniInt(length)/F4MiniInt(length)
	H4 *=  Histomoment (ampdist,1)*Histomoment (ampdist,3)/Histomoment (ampdist,4)
	Z4 = (F4MiniInt(length))^3/(F3MiniInt(length))^4
	Z4 *=  (Histomoment (ampdist,4))^3/(Histomoment (ampdist,3))^4
	MiniVar = 1000* Histomoment (ampdist,2)*F2MiniInt(length)
	Print  "Hs, Zs,     H4, Z4,    MiniVar = ",Hs, Zs,"	", H4, Z4,"	", minivar
end

Function Histomoment (distrib,moment)
	wave  distrib	//distrib is the histogram, and moment is a number indicating
					//the order of the moment
					//the function assumes the IGOR convention of histograms to hold,
					//which implies that the mean x-value for a bin is  x+dltax/2
	Variable moment
	duplicate/O  distrib dum11
	Variable xhalfInterval=deltax(distrib)/2,  lastpoint = numpnts(distrib)-1, count
	integrate dum11
	count = dum11[lastpoint]	//the integral over the histo
	dum11 =(x+ xhalfInterval)^moment* distrib
	integrate dum11
	return  dum11[lastpoint]/count	
end
	
Function clip (distrib, thresh)
	wave  distrib	//wave to be clipped
	Variable Thresh	//clipping threshold
		//puts a NAN  into all values of a wave  for which  the value is larger than thresh
		//good for display purposes
	Variable i=0, lastpoint = Numpnts(distrib)
	do
		IF 	(distrib[i] > thresh)
			distrib[i] = nan
		endif
		i += 1
	while (i < lastpoint)
end
	
	

Function Findpoint (searchwave, yVal)
	wave searchwave
	Variable yVal
		//looks for first point, when  |searchwave| >= yVal; aborts at i = 20000????
	Variable i = -1
	do
		i +=1
	//print  abs(searchwave[i])
	while (abs(searchwave[i]) < yVal)
	return  i
end



Function 	Decon( Current,Rate,Resid,Tau1,alpha,tau2,beta, t_end,Flag)
	Wave Current,Rate,Resid
	Variable  Tau1,alpha,tau2,beta,t_end,Flag
	//  deconvolves 'Current'  as described by N&S2001a  assuming that 'Current ' is a sum of a mEPSC-
	 //generated current and  a term according to  (beta*Int (m(t')*  z(t-t')dt' )^n    where z(t) is  
	 //a diffusion kernel and m(t) is the deconvolution result (Rate). 'Resid'  is also a result wave, 
	// which contains the 'Residual Current. Tau1,  Tau2 and  alpha are parameters of the mEPSC
	//beta is a scaler for the contribution of the residual current; t_end is the end  of the analysis
	//D is the diff constant in um^2/sec; Dist is effective distance in um; the other parameters
	//of the diffusion model are global Variables
	//returns rate(t)*h as average 'rate' in Amps/sec over an interval of Time-to_peak
	//Resid is   the diffusion-related current //y as described above
	// If Flag = 5 no deconvolution is performed; only the residual current  is calculated on the basis of 
	//the wave Rate(in Amps/sec), which  inthis case is used  as an input  wave, and left unchanged
	//Otherwise (Flag <> 5) the deconvolution  is performed  according to the Appendix of N&S2001a
	//Theory: If  yz id the diffusion related current, we have: rate = corr1....+ [d/dt(y-yz) + (y-yz)/Tau1
	// - alpha*A0*(1/Tau1 - 1/Tau2)*Int2]/(1+PeakInt/Tau1)  
	//Int2 is actually  calculated as h*Int2
	
	Duplicate/O Current, Int2,Zet,ZetC,RateC,mini,miniInt
	Variable i,sum =0,Num,factor,exp1,exp2,Del,k,deriv, meancurrent,Int3,start,diffrise
	Variable A0,PeakInt,RiseCorr, Corr1,DelMin, T1Star,T2Star, T3Star, ThisY, LastY
	NVar D=d
	NVar dist = dist
	NVar risetime = risetime
	NVar n = n	//The exponent of the postsynaptic power function
	NVar nd = nd
	NVar m = m	//The compression factor  for speeding up the calculation Int3;  large m makes deconvolution faster

	Del = DeltaX(Current)
	Num = x2pnt ( Current, t_end ) 
	Int2 =0; Int3 =0
	Resid =0
	ZetC =0; RateC =0
	IF (Flag !=5)
		Rate = 0 	//start with zero unless we want to calculate diffuse part only
	Endif
	
		//work on the mini and calculate all the numbers needed below
	A0 = MakeMini (current,risetime,Tau1,Tau2,alpha)	//A0 is the  extrapolation to zero
	Wavestats/Q Mini	//we need the time of the peak, V_maxLoc
	MiniInt = Mini; Integrate MiniInt
	PeakInt = MiniInt(V_maxLoc)	//the  Integral at the maximum
	DelMin =  V_maxloc/Del			//the point number of the maximum
	T1Star = findpoint(Mini,0.5)	//The Time of half-rise (at which the derivative of the mEPSC is equal to its
									//equal to its mean over the rising phase
	T2Star = findpoint(MiniInt,0.5*PeakInt)	//The theoretically correct value
			//not used, see below for smoother decay
	T3Star = (V_maxloc + 0.0* Risetime)/Del //this is better than the  sum of the two
	//Print "Mini times for Max, t*, t**,t***", DelMin, T1Star, T2Star, T3Star
	
		//now calculate the correction factors
	RiseCorr = PeakInt/Tau1 	//a  correction regarding the finite rising phase (from y)
	Corr1 = A0*(risetime/Tau1-1)*exp(-V_maxLoc/risetime)//also rising phase, relatively 
									//small, compared to above
	exp1 = exp(-V_maxloc/Tau2)
	exp2 = exp(-Del/Tau2)  
	factor = alpha*A0*(1/Tau2  - 1/Tau1 )	//
	//print "Risecorr in deconvol,  A0,factor, Corr1:  ", RiseCorr,A0,factor, Corr1

		//work on the diffusion kernel, both  the normal (Zet) and timecompressed version (ZetC)
		//we mprovide a compressed version for more rapid calculation below
	Zet =   exp(-Dist*Dist/(4*D*x))/(4*Pi*D*(x)^nd)		//the full diffusion kernel
	ZetC = Zet
	Compress (ZetC,m)		//ZrtC is the compressed diffusion kernel (1 point for m points)

		//Determine the best startpoint (start) for compressed calculation
	diffrise = Dist^2/4/D/del
	If  (m > diffrise/4)
		Print "m is too large!!!"
	endif
	start = 1.5*Diffrise
	If (start > 6*m)	//delay need not be larger than  6*m
  		start = 6*m
	EndIf	


	i =1	
		//here starts the iteration loop for calculating diffuse current and deconvolution
	do
		If (beta != 0)	//calculate the diffuse current 
			Int3 = DoIntegralC (Rate,RateC,Zet,ZetC,i-1,m,start)//	returns Int3, the convolution sum  of  the 	
						//diffuse current; works with compressed version
			//Int3 = DoIntegral (Rate,Zet,i)	//Alternatively the more accurate full sum
			IF (Int3<0)
				resid[i] =  -(-beta*Int3*del)^n 
			EndIf
		EndIf

		IF (Flag ==5)
				//leave rate as is. in case only the diffuse current is supposed to be calculated
		Else	//calculate a new value for rate; it is based on the mean current and mean deriv. at points i and i-1
				// and is entered  at i-T1Star, because it represents  the mean between i-DelMin  and i
			ThisY =  current[i]-resid[i]
			LastY =  current[i-1]-resid[i-1]
			deriv = (ThisY - LastY)/del
			meancurrent = (ThisY + LastY)/2
			Int2[i] =  Rate[i-DelMin]*Del*exp1+ exp2*Int2[i -1]	//recursive calculation of Int2; index i containing the 
				//integral relevant for the rate-calculation  performed  at that index 
			Rate[i-T1Star,i]   = Deriv + meancurrent /Tau1+ factor*Int2[i] + Corr1*Rate[i-T3Star] 
			Rate[i-T1Star,i] /= (1 + risecorr)
				//we fill the most recent T1Star+1 values with the new one, such that we can calculate  'surround'
		Endif

				//calculate a condensed form of the rate (RateC)  m points condensed int 1
		If  (trunc((i-T1Star+1)/m) == (i-T1Star+1)/m)	//take average whenever m new permanent values are available
			k =0
			sum =0
			do
				sum += Rate[i-T1Star+1+k-m]
				k +=1
			while (k < m)
			RateC[(i-T1Star+1)/m-1] = sum/m	//we fill the RateC from index 0, the first entry 
					//representing the average rate from 0 to m-1
		endif

		i +=1
	while  (i<Num)

end		//Function Decon




Function  DoIntegralC (Rate,RateC,Func,FuncC,i,m,delay)
	Wave Rate,RateC,Func,FuncC
	Variable i,m,delay
			//this calculates the convolution sum for index i, based on all values up to i-1
			//we go backwards from (k= )  i-1, to  0     rate[k]*Func[i-k]
			//so first product is  rate[i-1]*Func[1], then  rate[i-2]*Func[2] ....
			//we want to switch at i= start  to a condensed calculation for higher speed
			//using condensed functions RateC and FuncC,  condensing factor m
			//'start'  is calculate from delay, see below
	
	
	Variable result=0, k=i-1,start,i2,k2,realik,truncik,alfa
	
	start = m*trunc((k-delay)/m) -1	//This is the startindex of the condensed calculation
					//it has to be an integer multiple of m minus1
					// it turned out to be bad when the start of the course calculation is too early
	IF (start <0)
	start =-1
	endif


	do				//add up contributions up to one above the start point for condensed calculation
	result += rate[k]*Func[i-k]
	k -=1
	while (k>start)	
	
		//now  k is equal to start and  positions [0 , start]  (of the rate) are left over  to sum up
		//we do this with the compressed (averaged) data, remembering that  the average of 0 to m-1
		// is stored in 0

	k2 = (k +1)/m -1	//k2 is now the appropriate index for the condensed rate  trace, which
		//contains  the average rate between  k and (k - m);  we want to multiply this with a 
		//condensed  trace FuncC, which represents the average  of  i-k  and i-k+m;  but we do not 
		//have such an average,  we rather have averages  of points 0 to m-1, or more general, of the m-truncated 
		// (i-start) to the next , and 
							
	realik =  (i-k)/m		// this is the desired non-integer  i-k value of condensed trace
	truncik = trunc(realik)	//this is the  truncated one
		//we construct the  FuncC[realik] as  (1-alfa)*FuncC[truncik] +alfa*FuncC[truncik+1],  with:
	alfa = (realik-truncik)

	IF (k2>=0)	//here starts the loop of the remaining summing up
		do
			result +=  m*rateC[k2]*((1-alfa)*FuncC[truncik] +alfa*FuncC[truncik+1])
			k2 -=1
			truncik +=1
		while (k2>=0)	
	EndIf
	
	return result	
end



Function  DoIntegral (Rate,Func,i)	//similar to DoIntegralC, without condesing
	Wave Rate,Func
	Variable i
	Variable k=i-1,result=0
	do				//add up contributions 
	result += rate[k]*Func[i-k]
	k -=1
	while (k>=0)	
	return result
end



Function 	Compress (SWave,m)		//Replace by compressed average for all full mtupels
	Wave SWave
	Variable m
	Variable i=0,k=0,nw,sum =0
	nw = Numpnts (SWave)
	do
		k =0
		sum =0
		do
			sum += SWave[i+k]
			k +=1
		while (k < m)
		SWave[i/m] = sum/m
		i += m
	while  (i <= (nw-m))	
End
	


Function 	CalcVarN (Current,Var,Filtered,highP,lowP,Begx, Endx)
	wave Current,Var,Filtered	//InputWave, resultwave, and filtered wave
	Variable highP,lowP,Begx, Endx	//filgtering windows (sec);beginning and end of analysis (sec)
			//  Variance calculation after   bandpass filtering  the input wave between Begx and Endx
			//lowP is a  time window for low  pass filtering ; highP  one for highpass filtering
	
	Variable i,diff,BegP,EndP,DelP,DelP1,DelP2,Halfw
	BegP = x2Pnt(Current,BegX)
	EndP = x2Pnt(Current,EndX)

	// First, Filter the trace --> into wave 'Filtered'
	Filtered = Current
	BandFilter (Filtered,lowP,highP)
	Var = 0
	i=BegP
	do
		diff  =Filtered[i]
		var[i] = diff*diff
		i +=1
	while (i<= endP)
End


Function 	CalcCumulants (Current,Var,Skew,Fourth,Filtered,DelTa,Del1,Begx, Endx)
	wave Current,Var,Skew,Fourth,Filtered	//InputWave,  resultwaves
	Variable DelTa,Del1,Begx, Endx	//averaging window sizes DelTa and Del1  in  units of sec
									//for high- and lowpass;beginning and end (sec)
			// This is a misnomer: The Function calculates the moments, after high- and lowpass 
			//filtering (i.e. with mean = zero) The 4th cumulant is calculated in AvCumulants
			//  In this version the current is  bandpass filtered  before  calculating variance
			//Del1 is a  time windows for low  pass filtering  
			//no smoothing is done  - all smoothing is done in  AvCumulants
	
	Variable i,diff,BegP,EndP
	BegP = x2Pnt(Current,BegX)
	EndP = x2Pnt(Current,EndX)

	// First, Filter the trace --> into wave 'Filtered'
	Filtered = Current
	BandFilter (Filtered,Del1,DelTa)


	i=BegP
	do
		diff  =Filtered[i]
		var[i] = diff*diff
		Skew[i] = diff*diff*diff
		Fourth[i] = var[i]*var[i]
		i +=1
	while (i<= endP)
End


Function RunSeriesResComp (InWave, ResultWave,R_s,C_m,fraction)
	wave InWave, ResultWave
	Variable R_s,C_m,fraction
		//prepares waves for seriesresitance compensation and calls the function (see below)
		//Both Inwave and resultwaves have to exist e.g. the resultwave being a duplicate of InWave
	NVar V_hold = V_hold
	NVar V_reversal= V_reversal
	ResultWave =  InWave
	SeriesresistanceComp (ResultWave, R_s,C_m,fraction,V_hold,V_reversal)
end
	


Function SeriesresistanceComp (I_wave, R_s,C_m,fraction,V_hold,V_reversal)
	wave I_wave		//the data wave
	Variable R_s,C_m,fraction,V_hold,V_reversal
			// The function replaces I-wave by its series-compensated version;
			// the value at i is replaced by the average at i and i+1
			//R_s is in ohms, C_m in Farads, fraction is the fraction to be compensated
			//if R_s was 5 MOhm in the experiment and if it was 50% hardware compensated
			//then R_s = 2.5e6 has to be entered and f=1 for a complete overall compensation
			//The routine, similarly to that published by  Traynelis J.Neurosc.Meth. 86:25, compensates
			//the frequency response, assuming a single R_s*C_m -timeconstant (at constant V-hold) and
			//a threecomponent equivalent circuit for the pipette cell assembly with  R_s, C_m, R_m
			//Theory: V_h =R_s*I+U_m;    I_r is membrane resistive  current, I  is total current
			//I_r= I-I_c=I-C_m*dU_m/dt=I+C_m*R_s*dI/dt (because R_s*I+U_m = const.)
			//G_m=I_r/ U_m = (I+C_m*R_s*dI/dt)/ (V_h-R_s*I)
			//For complete correction (fraction =1) : I_corr = V_h*(I+C_m*R_s*dI/dt)/ (V_h-R_s*I)
	Variable numpoints = numpnts(I_wave), sampInt = deltax(I_wave), i, voltage, tau
	Variable Current, derivative, denominator
	Variable R_m, tau_corr
	
	voltage = V_hold - V_reversal
	tau = R_s*C_m
	tau_corr = R_s*C_m*(1-fraction)

		//First point:   (we have to calculate this seperately, because we need the value at i-1 below)
	denominator = voltage - R_s*fraction*I_wave[0]
	IF (denominator != 0)
		I_wave[0] *= (voltage/denominator)
	endif
	
	i=1
	do				//this is the loop doing the correction  for all other points
					//first calculate R_m for zero series resistance under the assumptions
					// that  U_m + U_Rs = const = voltage
		current = (I_wave[i+1] +I_wave[i])/2	//The in between(mean) value
		derivative =  (I_wave[i+1] -I_wave[i])/sampInt
		denominator = current+tau*derivative
		IF (denominator != 0)
			R_m = (voltage - R_s*current)/denominator	//calculate the true R_m
		ELSE
			R_m +=0	//leave as is
		Endif
			// Now calculate current for new series resitance
		denominator = (R_m + (1-fraction)*R_s)*(1+tau_corr/sampInt)
		IF (denominator != 0)
			I_wave[i] = tau_corr/(tau_corr+sampInt)*I_wave[i-1] + voltage/denominator
		ELSE
			I_wave[i] = I_wave[i-1]	//old value
		endif
		
		i +=1
	while  (i< numpoints)
	
end




Function 	MakeMini ( trace,Taurise,Tau1,Tau2,alpha)
	Wave  trace
	Variable Taurise,Tau1,Tau2,alpha
	//Makes a mini, using trace as template with max amplitude  = 1; returns a factor which is 
	//the extrapolated value to t=0
	// We need this  for scaling the  deconvolution
	//*****mini(0) has to be 0 !!!   (for later use in Abscalibration)
	Duplicate/O trace mini
	Variable  a0=-1,a1,a2,result	// for start point at zero
	a1 = 1-alpha
	a2 = alpha
	Mini = a0*exp(-x/Taurise) + a1*exp(-x/Tau1) + a2*exp(-x/Tau2)
	Wavestats/Q Mini
	//Print v_max
	Mini /=  V_max	// This is normalized to 1
	Return 1/V_max //  extrapolation to time 0 (for Decon)
end



Function 	SubOffset (InWave, t_start)
	Wave Inwave
	Variable t_start
	//  This function subtracts from the Inwave the mean between 0 and  t_start
	//  We assume that x start at zero
	
	Variable i=0,  sum=0,Del
	Del = DeltaX(Inwave)
	do
		sum += InWave[i]
		i +=1
	while  (Del*i < t_start)
	sum /=i
	Print "Offset of current trace = ", sum
	Inwave = Inwave - sum
end	


Function NPoints (Current,Del1,flag)
	wave current
	variable Del1,flag	// returns an  number of points, covering about the stretch Del1
					//  of wave Current
					// If flag >0 this number is uneven
	
	Variable DelP
	DelP = Trunc(Del1/DeltaX(Current))
	IF (Flag >0)
	If  (Trunc(DelP/2) == (DelP/2))
		DelP +=1		//DelP has to be uneven
	Endif
	Endif
	return DelP
End


Function DoubleSmooth (Trace, window)
	wave trace
	Variable window	//this performs two smoothing operations, once  with 'window' 
			//and once with 2/3 window  - to discriminate against 1.5th harmonic
			//window is in sec
			Variable WindowP2, WindowP
	
			WindowP = NPoints (Trace, window,1)
			WindowP2 = NPoints (Trace, window*0.8,1)
			//Print  WindowP, WindowP2
			Smooth/B WindowP, Trace
			If  (WindowP2 != WindowP)
				Smooth/B WindowP2, Trace
			ELSE
				//Print "Only single smoothing, because window has too few points"
			Endif
			If  (WindowP2 == 1)
				//Print "Second smoothing ineffective (P=1)"
			Endif
End


Function BandFilter (trace,T1,T2)
	wave trace
	Variable T1,T2
	//Do  Filtering with Lowpass window T1 and highpass T2
	Variable HalfW2,T3
	Halfw2 = (NPoints (trace, T2,1)-1)/2
	DoubleSmooth (Trace,T1)	//Low pass filtering
	Duplicate/O   Trace BandPassWave
	DoubleSmooth (BandPassWave, T2)
	trace    -= BandPassWave[p-HalfW2]	//high pass  filtering
	//second cycle of highpassfiltering
	T3 = T2*8
	BandPassWave = trace
	DoubleSmooth (BandPassWave, T3)
	Halfw2 = (NPoints (trace, T3/1.6,1)-1) 
	//Print Halfw2
	trace    -= BandPassWave[p+Halfw2]	//Alternative for  hardware-like filtering
	
END


Function 	Calcmean (InWave, t_start, t_end,flag)
	Wave Inwave
	Variable t_start,t_end,flag
	//  This functioncalculates  mean  of the Inwave , starting at   t_start , ending t_end
	// Flag: it prints variance if flag = 1
	
	Variable p_start,p_end,Delta,Value
	Delta = DeltaX(Inwave)
	If (trunc(t_start/delta)== t_start/delta)
		p_start =t_start/delta
	ELSE
	p_start = trunc(t_start/delta) +1
	EndIf
	p_end = trunc(t_end/delta) 
	Value = CalcmeanP (InWave, p_start, p_end,flag)
	//Print "CalcMean = ", Value, t_start
	Return Value
end	


Function 	CalcmeanP (InWave, p_start, p_end, flag)
	Wave Inwave
	Variable p_start,p_end, flag
	//  This functioncalculates  mean  of the Inwave , starting at  pont p_start , ending p_end
	// Flag: it prints variance if flag = 1
	
	Variable xVal=p_start, i=0, sum=0,sum2=0
	do
	i +=1
		sum += InWave[xVal]
		sum2 += InWave[xVal]*InWave[xVal]
		xVal +=  1
	while  (xVal < p_end)
	sum /=i
	If (i>1)
		sum2 = (sum2 - i*sum^2)/(i-1)
		If (flag ==1)
		Print  "Variance on window =  ", sum2,"    S.D. =  ",sqrt(sum2),"Mean^2/Var      ", sum^2/sum2
		Endif
	Endif
	//Print "Mean during interval of length:    ",t_end-t_start ," = ",sum
	Return sum
end	


Function MeanFromHist (StandardMini)
	Wave StandardMini
		//  The convention is that bin 0 contains entries with values between  0 and  deltaX
	Variable classes, AvAmp = 0, TotalN = 0, Del,i =0,max
	Del = deltax(standardMini)
	max = numpnts (StandardMini)
	do
	AvAmp += (i + 0.5)*Del*standardMini [i]
	TotalN  += standardMini [i]
	i +=1
	while (i <max)
	//Print  AvAmp/Del, TotalN, Del
	AvAmp  /= TotalN	//This is now the mean mini amplitude
	return AvAmp
end


Function Blank (Source, ComparWav, limit)
	wave Source, ComparWav
	Variable limit
	
	//Blank out too small values
	Variable num, i =0
	num = numpnts(Source)
	do
		If  ( ComparWav[i] < limit )
			Source[i] = NAN
		endif
		i +=1
	while ( i< num)
end



Function 	ChangeColour (ColourWave)
	Wave ColourWave
	Variable red = ColourWave[0], green = ColourWave[1], blue = ColourWave[2]
	
	If (red>= 15000)
		red -= 15000
		green +=15000
	Else
		If (green>= 15000)
			green -= 15000
			blue +=15000
		ELse
			blue -= 15000
			red += 15000
		Endif
	EndIf
	ColourWave[0]  = red; ColourWave[1]  = green; ColourWave[2]  = blue
End
		
Function Median (Swave, t_s, t_e)
	Wave SWave
	Variable t_s,t_e
		//calculates median of Swave between  t_s  and t_e
	
	Variable result
	Duplicate/R=(t_s,t_e ) SWave tempMWave
	Sort  tempMWave, tempMWave
	SetScale/P x 0,1,tempMWave
	result = tempMWave((numpnts(tempMWave)-1)/2)
	KillWaves tempMWave
	
	return result
end


Function LSCoefficient (trace, MeanTrace, t1,t2)
	wave trace,meantrace
	variable t1,t2
	//calculates the coefficient for subtracting the mean wave from a given wave for minimizing 
	//meansquare deviation in  window t1,t2
	// calculating  'a' for minimization of  Sum( Yi - a*Ymi +b)^2,  where Ymi is a mean trace
	// similar to optimal scaling to template of  Clements and Bekker Biophys. J. 73, p220
	
	variable Sim=0, Sii=0,Smm=0,Si=0,Sm=0,p1 = x2pnt(trace,t1),p2 = x2pnt(trace,t2)
	Variable yi,ym,N = p2-p1+1, result
	do
		yi = trace[p1]
		ym = Meantrace[p1]
		Sim += yi*ym
		Smm += ym*ym
		Sii += yi*yi 
		Si += yi
		Sm +=ym
		p1+=1
	while (p1<=p2)
	result = (N*Sim - Si*Sm)/(N*Smm - Sm*Sm)
	//print result
	return result
end
	
